@@ -26,6 +26,8 @@ this.atomIndex0 = 0;
 this.joinList = null;
 this.mapRtoA = null;
 this.deleteAtoms = null;
+this.moleculeID = null;
+this.htModelAtomMap = null;
 Clazz.instantialize (this, arguments);
 }, J.adapter.readers.xml, "XmlCmlReader", J.adapter.readers.xml.XmlReader);
 Clazz.prepareFields (c$, function () {
@@ -37,32 +39,29 @@ Clazz.makeConstructor (c$,
 function () {
 Clazz.superConstructor (this, J.adapter.readers.xml.XmlCmlReader, []);
 });
-Clazz.overrideMethod (c$, "getDOMAttributes", 
-function () {
-return  Clazz.newArray (-1, ["id", "title", "label", "name", "x3", "y3", "z3", "x2", "y2", "isotope", "elementType", "formalCharge", "atomId", "atomRefs2", "order", "atomRef1", "atomRef2", "dictRef", "spaceGroup"]);
-});
 Clazz.overrideMethod (c$, "processStartElement", 
-function (name) {
+function (name, nodeName) {
 if (!this.processing) return;
 this.processStart2 (name);
-}, "~S");
+}, "~S,~S");
 Clazz.defineMethod (c$, "processStart2", 
 function (name) {
 name = name.toLowerCase ();
+var val;
 switch (this.state) {
 case 0:
 if (name.equals ("molecule")) {
+this.moleculeID = this.atts.get ("id");
 this.state = 6;
 this.haveMolecule = true;
-if (this.moleculeNesting == 0) {
-this.createNewAtomSet ();
-}this.moleculeNesting++;
+if (this.moleculeNesting == 0) this.createNewAtomSet ();
+this.moleculeNesting++;
 } else if (name.equals ("crystal")) {
 this.state = 2;
 } else if (name.equals ("symmetry")) {
 this.state = 17;
-if (this.atts.containsKey ("spaceGroup")) {
-this.localSpaceGroupName = this.atts.get ("spaceGroup");
+if ((val = this.atts.get ("spacegroup")) != null) {
+this.localSpaceGroupName = val;
 } else {
 this.localSpaceGroupName = "P1";
 this.parent.clearUnitCell ();
@@ -82,13 +81,13 @@ this.scalarTitle = this.atts.get ("title");
 this.getDictRefValue ();
 } else if (name.equals ("symmetry")) {
 this.state = 4;
-if (this.atts.containsKey ("spaceGroup")) {
-this.localSpaceGroupName = this.atts.get ("spaceGroup");
+if ((val = this.atts.get ("spacegroup")) != null) {
+this.localSpaceGroupName = val;
 for (var i = 0; i < this.localSpaceGroupName.length; i++) if (this.localSpaceGroupName.charAt (i) == '_') this.localSpaceGroupName = this.localSpaceGroupName.substring (0, i) + this.localSpaceGroupName.substring ((i--) + 1);
 
 }} else if (name.equals ("cellparameter")) {
-if (this.atts.containsKey ("parameterType")) {
-this.cellParameterType = this.atts.get ("parameterType");
+if ((val = this.atts.get ("parametertype")) != null) {
+this.cellParameterType = val;
 this.setKeepChars (true);
 }}break;
 case 18:
@@ -113,59 +112,59 @@ this.embeddedCrystal = true;
 } else if (name.equals ("molecule")) {
 this.state = 6;
 this.moleculeNesting++;
-} else if (name.equalsIgnoreCase ("join")) {
+} else if (name.equals ("join")) {
 var order = -1;
 this.tokenCount = 0;
-if (this.atts.containsKey ("atomRefs2")) {
-this.breakOutTokens (this.atts.get ("atomRefs2"));
-if (this.atts.containsKey ("order")) order = this.parseBondToken (this.atts.get ("order"));
+if ((val = this.atts.get ("atomrefs2")) != null) {
+this.breakOutTokens (val);
+if ((val = this.atts.get ("order")) != null) order = this.parseBondToken (val);
 if (this.tokenCount == 2 && order > 0) this.joinList.addLast ( Clazz.newArray (-1, [this.tokens[0], this.tokens[1], "" + order]));
 }} else if (name.equals ("bondarray")) {
 this.state = 10;
 this.bondCount = 0;
-if (this.atts.containsKey ("order")) {
-this.breakOutBondTokens (this.atts.get ("order"));
+if ((val = this.atts.get ("order")) != null) {
+this.breakOutBondTokens (val);
 for (var i = this.tokenCount; --i >= 0; ) this.bondArray[i].order = this.parseBondToken (this.tokens[i]);
 
-}if (this.atts.containsKey ("atomRef1")) {
-this.breakOutBondTokens (this.atts.get ("atomRef1"));
+}if ((val = this.atts.get ("atomref1")) != null) {
+this.breakOutBondTokens (val);
 for (var i = this.tokenCount; --i >= 0; ) this.bondArray[i].atomIndex1 = this.getAtomIndex (this.tokens[i]);
 
-}if (this.atts.containsKey ("atomRef2")) {
-this.breakOutBondTokens (this.atts.get ("atomRef2"));
+}if ((val = this.atts.get ("atomref2")) != null) {
+this.breakOutBondTokens (val);
 for (var i = this.tokenCount; --i >= 0; ) this.bondArray[i].atomIndex2 = this.getAtomIndex (this.tokens[i]);
 
 }} else if (name.equals ("atomarray")) {
 this.state = 7;
 this.aaLen = 0;
 var coords3D = false;
-if (this.atts.containsKey ("atomID")) {
-this.breakOutAtomTokens (this.atts.get ("atomID"));
+if ((val = this.atts.get ("atomid")) != null) {
+this.breakOutAtomTokens (val);
 for (var i = this.tokenCount; --i >= 0; ) this.atomArray[i].atomName = this.tokens[i];
 
-}if (this.atts.containsKey ("x3")) {
+}if ((val = this.atts.get ("x3")) != null) {
 coords3D = true;
-this.breakOutAtomTokens (this.atts.get ("x3"));
+this.breakOutAtomTokens (val);
 for (var i = this.tokenCount; --i >= 0; ) this.atomArray[i].x = this.parseFloatStr (this.tokens[i]);
 
-}if (this.atts.containsKey ("y3")) {
-this.breakOutAtomTokens (this.atts.get ("y3"));
+}if ((val = this.atts.get ("y3")) != null) {
+this.breakOutAtomTokens (val);
 for (var i = this.tokenCount; --i >= 0; ) this.atomArray[i].y = this.parseFloatStr (this.tokens[i]);
 
-}if (this.atts.containsKey ("z3")) {
-this.breakOutAtomTokens (this.atts.get ("z3"));
+}if ((val = this.atts.get ("z3")) != null) {
+this.breakOutAtomTokens (val);
 for (var i = this.tokenCount; --i >= 0; ) this.atomArray[i].z = this.parseFloatStr (this.tokens[i]);
 
-}if (this.atts.containsKey ("x2")) {
-this.breakOutAtomTokens (this.atts.get ("x2"));
+}if ((val = this.atts.get ("x2")) != null) {
+this.breakOutAtomTokens (val);
 for (var i = this.tokenCount; --i >= 0; ) this.atomArray[i].x = this.parseFloatStr (this.tokens[i]);
 
-}if (this.atts.containsKey ("y2")) {
-this.breakOutAtomTokens (this.atts.get ("y2"));
+}if ((val = this.atts.get ("y2")) != null) {
+this.breakOutAtomTokens (val);
 for (var i = this.tokenCount; --i >= 0; ) this.atomArray[i].y = this.parseFloatStr (this.tokens[i]);
 
-}if (this.atts.containsKey ("elementType")) {
-this.breakOutAtomTokens (this.atts.get ("elementType"));
+}if ((val = this.atts.get ("elementtype")) != null) {
+this.breakOutAtomTokens (val);
 for (var i = this.tokenCount; --i >= 0; ) this.atomArray[i].elementSymbol = this.tokens[i];
 
 }for (var i = this.aaLen; --i >= 0; ) {
@@ -175,14 +174,16 @@ this.addAtom (atom);
 }
 } else if (name.equals ("formula")) {
 this.state = 12;
+} else if (name.equals ("association")) {
+this.state = 19;
 }break;
 case 10:
 if (name.equals ("bond")) {
 this.state = 11;
 var order = -1;
 this.tokenCount = 0;
-if (this.atts.containsKey ("atomRefs2")) this.breakOutTokens (this.atts.get ("atomRefs2"));
-if (this.atts.containsKey ("order")) order = this.parseBondToken (this.atts.get ("order"));
+if ((val = this.atts.get ("atomrefs2")) != null) this.breakOutTokens (val);
+if ((val = this.atts.get ("order")) != null) order = this.parseBondToken (val);
 if (this.tokenCount == 2 && order > 0) {
 this.addNewBond (this.tokens[0], this.tokens[1], order);
 }}break;
@@ -192,32 +193,32 @@ this.state = 8;
 this.atom =  new J.adapter.smarter.Atom ();
 this.parent.setFractionalCoordinates (false);
 var id = this.atts.get ("id");
-if (this.atts.containsKey ("name")) this.atom.atomName = this.atts.get ("name");
- else if (this.atts.containsKey ("title")) this.atom.atomName = this.atts.get ("title");
- else if (this.atts.containsKey ("label")) this.atom.atomName = this.atts.get ("label");
+if ((val = this.atts.get ("name")) != null) this.atom.atomName = val;
+ else if ((val = this.atts.get ("title")) != null) this.atom.atomName = val;
+ else if ((val = this.atts.get ("label")) != null) this.atom.atomName = val;
  else this.atom.atomName = id;
 if (!this.checkedSerial) {
 this.isSerial = (id != null && id.length > 1 && id.startsWith ("a") && JU.PT.parseInt (id.substring (1)) != -2147483648);
 this.checkedSerial = true;
 }if (this.isSerial) this.atom.atomSerial = JU.PT.parseInt (id.substring (1));
-if (this.atts.containsKey ("xFract") && (this.parent.iHaveUnitCell || !this.atts.containsKey ("x3"))) {
+if ((val = this.atts.get ("xfract")) != null && (this.parent.iHaveUnitCell || !this.atts.containsKey ("x3"))) {
 this.parent.setFractionalCoordinates (true);
-this.atom.set (this.parseFloatStr (this.atts.get ("xFract")), this.parseFloatStr (this.atts.get ("yFract")), this.parseFloatStr (this.atts.get ("zFract")));
-} else if (this.atts.containsKey ("x3")) {
-this.atom.set (this.parseFloatStr (this.atts.get ("x3")), this.parseFloatStr (this.atts.get ("y3")), this.parseFloatStr (this.atts.get ("z3")));
-} else if (this.atts.containsKey ("x2")) {
-this.atom.set (this.parseFloatStr (this.atts.get ("x2")), this.parseFloatStr (this.atts.get ("y2")), 0);
-}if (this.atts.containsKey ("elementType")) {
-var sym = this.atts.get ("elementType");
-if (this.atts.containsKey ("isotope")) this.atom.elementNumber = ((this.parseIntStr (this.atts.get ("isotope")) << 7) + J.api.JmolAdapter.getElementNumber (sym));
+this.atom.set (this.parseFloatStr (val), this.parseFloatStr (this.atts.get ("yfract")), this.parseFloatStr (this.atts.get ("zfract")));
+} else if ((val = this.atts.get ("x3")) != null) {
+this.atom.set (this.parseFloatStr (val), this.parseFloatStr (this.atts.get ("y3")), this.parseFloatStr (this.atts.get ("z3")));
+} else if ((val = this.atts.get ("x2")) != null) {
+this.atom.set (this.parseFloatStr (val), this.parseFloatStr (this.atts.get ("y2")), 0);
+}if ((val = this.atts.get ("elementtype")) != null) {
+var sym = val;
+if ((val = this.atts.get ("isotope")) != null) this.atom.elementNumber = ((this.parseIntStr (val) << 7) + J.api.JmolAdapter.getElementNumber (sym));
 this.atom.elementSymbol = sym;
-}if (this.atts.containsKey ("formalCharge")) this.atom.formalCharge = this.parseIntStr (this.atts.get ("formalCharge"));
+}if ((val = this.atts.get ("formalcharge")) != null) this.atom.formalCharge = this.parseIntStr (val);
 }break;
 case 11:
-if (this.atts.containsKey ("builtin")) {
+if ((val = this.atts.get ("builtin")) != null) {
 this.setKeepChars (true);
 this.state = 14;
-this.scalarDictValue = this.atts.get ("builtin");
+this.scalarDictValue = val;
 }break;
 case 8:
 if (name.equals ("scalar")) {
@@ -225,10 +226,10 @@ this.state = 9;
 this.setKeepChars (true);
 this.scalarTitle = this.atts.get ("title");
 this.getDictRefValue ();
-} else if (this.atts.containsKey ("builtin")) {
+} else if ((val = this.atts.get ("builtin")) != null) {
 this.setKeepChars (true);
 this.state = 13;
-this.scalarDictValue = this.atts.get ("builtin");
+this.scalarDictValue = val;
 }break;
 case 9:
 break;
@@ -259,6 +260,9 @@ if (--this.moduleNestingLevel == 0) {
 if (this.parent.iHaveUnitCell) this.applySymmetryAndSetTrajectory ();
 this.setAtomNames ();
 }}break;
+case 19:
+if (name.equals ("association")) this.state = 6;
+break;
 case 2:
 if (name.equals ("crystal")) {
 if (this.embeddedCrystal) {
@@ -267,7 +271,7 @@ this.embeddedCrystal = false;
 } else {
 this.state = 0;
 }} else if (name.equals ("cellparameter") && this.keepChars) {
-var tokens = JU.PT.getTokens (this.chars);
+var tokens = JU.PT.getTokens (this.chars.toString ());
 this.setKeepChars (false);
 if (tokens.length != 3 || this.cellParameterType == null) {
 } else if (this.cellParameterType.equals ("length")) {
@@ -296,7 +300,7 @@ this.setKeepChars (false);
 this.state = 4;
 }break;
 case 18:
-var values = J.adapter.smarter.AtomSetCollectionReader.getTokensFloat (this.chars, null, 3);
+var values = J.adapter.smarter.AtomSetCollectionReader.getTokensFloat (this.chars.toString (), null, 3);
 this.parent.addPrimitiveLatticeVector (this.latticeVectorPtr, values, 0);
 this.latticeVectorPtr = (this.latticeVectorPtr + 1) % 3;
 this.setKeepChars (false);
@@ -355,29 +359,29 @@ case 9:
 if (name.equals ("scalar")) {
 this.state = 8;
 if ("jmol:charge".equals (this.scalarDictRef)) {
-this.atom.partialCharge = this.parseFloatStr (this.chars);
+this.atom.partialCharge = this.parseFloatStr (this.chars.toString ());
 } else if (this.scalarDictRef != null && "_atom_site_label".equals (this.scalarDictValue)) {
 if (this.atomIdNames == null) this.atomIdNames =  new java.util.Properties ();
-this.atomIdNames.put (this.atom.atomName, this.chars);
+this.atomIdNames.put (this.atom.atomName, this.chars.toString ());
 }}this.setKeepChars (false);
 this.scalarTitle = null;
 this.scalarDictRef = null;
 break;
 case 13:
 this.state = 8;
-if (this.scalarDictValue.equals ("x3")) this.atom.x = this.parseFloatStr (this.chars);
- else if (this.scalarDictValue.equals ("y3")) this.atom.y = this.parseFloatStr (this.chars);
- else if (this.scalarDictValue.equals ("z3")) this.atom.z = this.parseFloatStr (this.chars);
- else if (this.scalarDictValue.equals ("elementType")) this.atom.elementSymbol = this.chars;
+if (this.scalarDictValue.equals ("x3")) this.atom.x = this.parseFloatStr (this.chars.toString ());
+ else if (this.scalarDictValue.equals ("y3")) this.atom.y = this.parseFloatStr (this.chars.toString ());
+ else if (this.scalarDictValue.equals ("z3")) this.atom.z = this.parseFloatStr (this.chars.toString ());
+ else if (this.scalarDictValue.equals ("elementType")) this.atom.elementSymbol = this.chars.toString ();
 this.setKeepChars (false);
 break;
 case 14:
 this.state = 11;
 if (this.scalarDictValue.equals ("atomRef")) {
 if (this.tokenCount == 0) this.tokens =  new Array (2);
-if (this.tokenCount < 2) this.tokens[this.tokenCount++] = this.chars;
+if (this.tokenCount < 2) this.tokens[this.tokenCount++] = this.chars.toString ();
 } else if (this.scalarDictValue.equals ("order")) {
-var order = this.parseBondToken (this.chars);
+var order = this.parseBondToken (this.chars.toString ());
 if (order > 0 && this.tokenCount == 2) this.addNewBond (this.tokens[0], this.tokens[1], order);
 }this.setKeepChars (false);
 break;
@@ -429,7 +433,7 @@ return (this.isSerial ? a.substring (1) : a);
 }, "~S");
 Clazz.defineMethod (c$, "getDictRefValue", 
  function () {
-this.scalarDictRef = this.atts.get ("dictRef");
+this.scalarDictRef = this.atts.get ("dictref");
 if (this.scalarDictRef != null) {
 var iColon = this.scalarDictRef.indexOf (":");
 this.scalarDictValue = this.scalarDictRef.substring (iColon + 1);
@@ -437,7 +441,7 @@ this.scalarDictValue = this.scalarDictRef.substring (iColon + 1);
 Clazz.defineMethod (c$, "checkUnitCellItem", 
  function (tags, value) {
 for (var i = tags.length; --i >= 0; ) if (value.equals (tags[i])) {
-this.parent.setUnitCellItem (i, this.parseFloatStr (this.chars));
+this.parent.setUnitCellItem (i, this.parseFloatStr (this.chars.toString ()));
 return;
 }
 }, "~A,~S");
@@ -445,6 +449,7 @@ Clazz.defineMethod (c$, "addAtom",
  function (atom) {
 if ((atom.elementSymbol == null && atom.elementNumber < 0) || Float.isNaN (atom.z)) return;
 this.parent.setAtomCoord (atom);
+if (this.htModelAtomMap != null) this.htModelAtomMap.put (this.moleculeID + atom.atomName, atom);
 if (this.isSerial) this.asc.addAtomWithMappedSerialNumber (atom);
  else this.asc.addAtomWithMappedName (atom);
 }, "J.adapter.smarter.Atom");
@@ -521,9 +526,9 @@ throw  new IndexOutOfBoundsException ("bad bond attribute length");
 Clazz.defineMethod (c$, "createNewAtomSet", 
  function () {
 this.asc.newAtomSet ();
-var collectionName = null;
-if (this.atts.containsKey ("title")) collectionName = this.atts.get ("title");
- else if (this.atts.containsKey ("id")) collectionName = this.atts.get ("id");
+var val;
+if (this.htModelAtomMap != null) this.htModelAtomMap.put ("" + this.asc.iSet, this.moleculeID);
+var collectionName = ((val = this.atts.get ("title")) != null || (val = this.atts.get ("id")) != null ? val : null);
 if (collectionName != null) {
 this.asc.setAtomSetName (collectionName);
 }});
@@ -559,5 +564,6 @@ Clazz.defineStatics (c$,
 "MODULE", 15,
 "SYMMETRY", 17,
 "LATTICE_VECTOR", 18,
+"ASSOCIATION", 19,
 "unitCellParamTags",  Clazz.newArray (-1, ["a", "b", "c", "alpha", "beta", "gamma"]));
 });
