@@ -124,7 +124,7 @@ if (!isOK) throw  new NullPointerException ();
 Clazz.overrideMethod (c$, "renderExport", 
 function (gdata, modelSet, params) {
 var isOK;
-this.shapeManager.finalizeAtoms (false, true);
+this.shapeManager.finalizeAtoms (null, true);
 var exporter3D = this.vwr.initializeExporter (params);
 isOK = (exporter3D != null);
 if (!isOK) {
