@@ -40,6 +40,8 @@ this.defaultFontId = this.zeroFontId = this.vwr.gdata.getFont3DFSS ("SansSerif",
 this.defaultColix = 0;
 this.defaultBgcolix = 0;
 this.defaultOffset = JV.JC.LABEL_DEFAULT_OFFSET;
+this.defaultAlignment = 4;
+this.defaultPointer = 0;
 this.defaultZPos = 0;
 this.translucentAllowed = false;
 });
@@ -52,9 +54,11 @@ return;
 }if ("color" === propertyName) {
 var pid = J.c.PAL.pidOf (value);
 var colix = JU.C.getColixO (value);
-if (!this.setDefaults) for (var i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) this.setLabelColix (i, colix, pid);
+if (!this.setDefaults) {
+var n = this.checkColixLength (colix, bsSelected.length ());
+for (var i = bsSelected.nextSetBit (0); i >= 0 && i < n; i = bsSelected.nextSetBit (i + 1)) this.setLabelColix (i, colix, pid);
 
-if (this.setDefaults || !this.defaultsOnlyForNone) {
+}if (this.setDefaults || !this.defaultsOnlyForNone) {
 this.defaultColix = colix;
 this.defaultPaletteID = pid;
 }return;
@@ -62,8 +66,8 @@ this.defaultPaletteID = pid;
 if (this.strings == null) return;
 var val = (value).floatValue ();
 var scalePixelsPerMicron = (val == 0 ? 0 : 10000 / val);
-for (var i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) {
-if (this.strings.length <= i) continue;
+var n = Math.min (this.ac, this.strings.length);
+for (var i = bsSelected.nextSetBit (0); i >= 0 && i < n; i = bsSelected.nextSetBit (i + 1)) {
 var text = this.getLabel (i);
 if (text == null) {
 text = JM.Text.newLabel (this.vwr, null, this.strings[i], 0, 0, 0, scalePixelsPerMicron);
@@ -75,14 +79,17 @@ return;
 }if ("label" === propertyName) {
 this.setScaling ();
 var tokens = null;
+var nbs = this.checkStringLength (bsSelected.length ());
+if (this.defaultColix != 0 || this.defaultPaletteID != 0) this.checkColixLength (this.defaultColix, bsSelected.length ());
+if (this.defaultBgcolix != 0) this.checkBgColixLength (this.defaultBgcolix, bsSelected.length ());
 if (Clazz.instanceOf (value, JU.Lst)) {
 var list = value;
 var n = list.size ();
 tokens =  Clazz.newArray (-1, [null]);
-for (var pt = 0, i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) {
+for (var pt = 0, i = bsSelected.nextSetBit (0); i >= 0 && i < nbs; i = bsSelected.nextSetBit (i + 1)) {
 if (pt >= n) {
 this.setLabel (J.shape.Labels.nullToken, "", i, true);
-return;
+continue;
 }tokens[0] = null;
 this.setLabel (tokens, JS.SV.sValue (list.get (pt++)), i, true);
 }
@@ -92,6 +99,11 @@ tokens = (strLabel == null || strLabel.length == 0 ? J.shape.Labels.nullToken :
 for (var i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) this.setLabel (tokens, strLabel, i, true);
 
 }return;
+}if (propertyName.startsWith ("label:")) {
+this.setScaling ();
+this.checkStringLength (this.ac);
+this.setLabel ( Clazz.newArray (-1, [null]), propertyName.substring (6), (value).intValue (), false);
+return;
 }if ("clearBoxes" === propertyName) {
 this.labelBoxes = null;
 return;
@@ -99,13 +111,15 @@ return;
 return;
 }if ("bgcolor" === propertyName) {
 this.isActive = true;
-if (this.bsBgColixSet == null) this.bsBgColixSet =  new JU.BS ();
+if (this.bsBgColixSet == null) this.bsBgColixSet = JU.BS.newN (this.ac);
 var bgcolix = JU.C.getColixO (value);
-if (!this.setDefaults) for (var i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) this.setBgcolix (i, bgcolix);
+if (!this.setDefaults) {
+var n = this.checkBgColixLength (bgcolix, bsSelected.length ());
+for (var i = bsSelected.nextSetBit (0); i >= 0 && i < n; i = bsSelected.nextSetBit (i + 1)) this.setBgcolix (i, bgcolix);
 
-if (this.setDefaults || !this.defaultsOnlyForNone) this.defaultBgcolix = bgcolix;
+}if (this.setDefaults || !this.defaultsOnlyForNone) this.defaultBgcolix = bgcolix;
 return;
-}if (this.bsFontSet == null) this.bsFontSet =  new JU.BS ();
+}if (this.bsFontSet == null) this.bsFontSet = JU.BS.newN (this.ac);
 if ("fontsize" === propertyName) {
 var fontsize = (value).intValue ();
 if (fontsize < 0) {
@@ -124,9 +138,10 @@ if (this.setDefaults || !this.defaultsOnlyForNone) this.defaultFontId = fid;
 return;
 }if ("offset" === propertyName) {
 if (!(Clazz.instanceOf (value, Integer))) {
-if (!this.setDefaults) for (var i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) this.setPymolOffset (i, value);
+if (!this.setDefaults) {
+for (var i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) this.setPymolOffset (i, value);
 
-return;
+}return;
 }var offset = (value).intValue ();
 if (!this.setDefaults) for (var i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) this.setOffsets (i, offset);
 
@@ -134,9 +149,7 @@ if (this.setDefaults || !this.defaultsOnlyForNone) this.defaultOffset = offset;
 return;
 }if ("align" === propertyName) {
 var type = value;
-var hAlignment = 4;
-if (type.equalsIgnoreCase ("right")) hAlignment = 12;
- else if (type.equalsIgnoreCase ("center")) hAlignment = 8;
+var hAlignment = (type.equalsIgnoreCase ("right") ? 12 : type.equalsIgnoreCase ("center") ? 8 : 4);
 for (var i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) this.setHorizAlignment (i, hAlignment);
 
 if (this.setDefaults || !this.defaultsOnlyForNone) this.defaultAlignment = hAlignment;
@@ -168,14 +181,16 @@ var strLabelUNK = null;
 var tokensUNK = null;
 var strLabel;
 var tokens;
+var nstr = this.checkStringLength (bsSelected.length ());
+var bgcolix = this.defaultBgcolix;
+var nbg = this.checkBgColixLength (bgcolix, bsSelected.length ());
+var thisMad = (mode >= 0 ? 1 : -1);
 for (var i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) {
 var atom = this.atoms[i];
-if (this.formats == null || i >= this.formats.length) this.formats = JU.AU.ensureLengthS (this.formats, i + 1);
-if (this.strings != null && this.strings.length > i && this.strings[i] != null) {
-this.mads[i] = (mode == 0 && this.mads[i] < 0 || mode == 1 ? 1 : -1);
+if (i < nstr && this.strings[i] != null) {
+this.mads[i] = (mode == 1 || mode == 0 && this.mads[i] < 0 ? 1 : -1);
 } else {
-if (this.bsSizeSet == null) this.bsSizeSet =  new JU.BS ();
-this.strings = JU.AU.ensureLengthS (this.strings, i + 1);
+this.mads[i] = thisMad;
 if (atom.getGroup3 (false).equals ("UNK")) {
 if (strLabelUNK == null) {
 strLabelUNK = this.vwr.getStandardLabelFormat (1);
@@ -191,20 +206,12 @@ tokens = tokensPDB;
 }this.strings[i] = JM.LabelToken.formatLabelAtomArray (this.vwr, atom, tokens, '\0', null, this.ptTemp);
 this.formats[i] = strLabel;
 this.bsSizeSet.set (i);
-if ((this.bsBgColixSet == null || !this.bsBgColixSet.get (i)) && this.defaultBgcolix != 0) this.setBgcolix (i, this.defaultBgcolix);
-this.mads[i] = (mode >= 0 ? 1 : -1);
+if (i < nbg && !this.bsBgColixSet.get (i)) this.setBgcolix (i, this.defaultBgcolix);
 }this.setShapeVisibility (atom, this.strings != null && i < this.strings.length && this.strings[i] != null && this.mads[i] >= 0);
 }
 return;
-}if (propertyName.startsWith ("label:")) {
-this.setScaling ();
-this.setLabel ( new Array (1), propertyName.substring (6), (value).intValue (), false);
-return;
-}if ("textLabels" === propertyName) {
-this.setScaling ();
-var labels = value;
-for (var i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) this.setTextLabel (i, labels.get (Integer.$valueOf (i)), null);
-
+}if ("pymolLabels" === propertyName) {
+this.setPymolLabels (value, bsSelected);
 return;
 }if (propertyName === "deleteModelAtoms") {
 this.labelBoxes = null;
@@ -219,26 +226,48 @@ JU.BSUtil.deleteBits (this.bsFontSet, bsSelected);
 JU.BSUtil.deleteBits (this.bsBgColixSet, bsSelected);
 }this.setPropAS (propertyName, value, bsSelected);
 }, "~S,~O,JU.BS");
+Clazz.defineMethod (c$, "checkStringLength", 
+ function (n) {
+n = Math.min (this.ac, n);
+if (this.strings == null || n > this.strings.length) {
+this.formats = JU.AU.ensureLengthS (this.formats, n);
+this.strings = JU.AU.ensureLengthS (this.strings, n);
+if (this.bsSizeSet == null) this.bsSizeSet = JU.BS.newN (n);
+}return n;
+}, "~N");
+Clazz.defineMethod (c$, "checkBgColixLength", 
+ function (colix, n) {
+n = Math.min (this.ac, n);
+if (colix == 0) return (this.bgcolixes == null ? 0 : this.bgcolixes.length);
+if (this.bgcolixes == null || n > this.bgcolixes.length) this.bgcolixes = JU.AU.ensureLengthShort (this.bgcolixes, n);
+return n;
+}, "~N,~N");
+Clazz.defineMethod (c$, "setPymolLabels", 
+ function (labels, bsSelected) {
+this.setScaling ();
+var n = this.checkStringLength (this.ac);
+for (var i = bsSelected.nextSetBit (0); i >= 0 && i < n; i = bsSelected.nextSetBit (i + 1)) this.setPymolLabel (i, labels.get (Integer.$valueOf (i)), null);
+
+}, "java.util.Map,JU.BS");
 Clazz.defineMethod (c$, "setPymolOffset", 
  function (i, value) {
 var text = this.getLabel (i);
 if (text == null) {
-if (this.strings == null || this.strings.length <= i || this.strings[i] == null) return;
+if (this.strings == null || i >= this.strings.length || this.strings[i] == null) return;
 var fid = (this.bsFontSet != null && this.bsFontSet.get (i) ? this.fids[i] : -1);
 if (fid < 0) this.setFont (i, fid = this.defaultFontId);
 text = JM.Text.newLabel (this.vwr, javajs.awt.Font.getFont3D (fid), this.strings[i], this.getColix2 (i, this.atoms[i], false), this.getColix2 (i, this.atoms[i], true), 0, this.scalePixelsPerMicron);
-this.setTextLabel (i, text, this.formats[i]);
-if (text == null) return;
+this.setPymolLabel (i, text, this.formats[i]);
 }text.pymolOffset = value;
 }, "~N,~A");
 Clazz.defineMethod (c$, "setScaling", 
  function () {
 this.isActive = true;
-if (this.bsSizeSet == null) this.bsSizeSet =  new JU.BS ();
+if (this.bsSizeSet == null) this.bsSizeSet = JU.BS.newN (this.ac);
 this.isScaled = this.vwr.getBoolean (603979845);
 this.scalePixelsPerMicron = (this.isScaled ? this.vwr.getScalePixelsPerAngstrom (false) * 10000 : 0);
 });
-Clazz.defineMethod (c$, "setTextLabel", 
+Clazz.defineMethod (c$, "setPymolLabel", 
  function (i, t, format) {
 if (t == null) return;
 var label = t.getText ();
@@ -277,8 +306,6 @@ if (this.defaultFontId != this.zeroFontId) this.setFont (i, this.defaultFontId);
 Clazz.defineMethod (c$, "addString", 
  function (atom, i, label, strLabel) {
 this.setShapeVisibility (atom, label != null);
-if (this.strings == null || i >= this.strings.length) this.strings = JU.AU.ensureLengthS (this.strings, i + 1);
-if (this.formats == null || i >= this.formats.length) this.formats = JU.AU.ensureLengthS (this.formats, i + 1);
 var notNull = (strLabel != null);
 var isNew = (this.strings[i] == null);
 this.strings[i] = label;
@@ -321,10 +348,7 @@ if (this.colixes != null && ((text = this.getLabel (i)) != null)) text.colix = t
 }, "~N,~N,~N");
 Clazz.defineMethod (c$, "setBgcolix", 
  function (i, bgcolix) {
-if (this.bgcolixes == null || i >= this.bgcolixes.length) {
-if (bgcolix == 0) return;
-this.bgcolixes = JU.AU.ensureLengthShort (this.bgcolixes, i + 1);
-}this.bgcolixes[i] = bgcolix;
+this.bgcolixes[i] = bgcolix;
 this.bsBgColixSet.setBitTo (i, bgcolix != 0);
 var text = this.getLabel (i);
 if (text != null) text.bgcolix = bgcolix;
@@ -332,19 +356,23 @@ if (text != null) text.bgcolix = bgcolix;
 Clazz.defineMethod (c$, "setOffsets", 
  function (i, offset) {
 if (this.offsets == null || i >= this.offsets.length) {
-if (offset == 0) return;
-this.offsets = JU.AU.ensureLengthI (this.offsets, i + 1);
-}if (offset == 0) offset = JV.JC.LABEL_DEFAULT_OFFSET;
-this.offsets[i] = (this.offsets[i] & 63) | offset;
+if (offset == JV.JC.LABEL_DEFAULT_OFFSET) return;
+this.offsets = JU.AU.ensureLengthI (this.offsets, this.ac);
+}this.offsets[i] = (this.offsets[i] & 63) | offset;
 var text = this.getLabel (i);
 if (text != null) text.setOffset (offset);
 }, "~N,~N");
 Clazz.defineMethod (c$, "setHorizAlignment", 
  function (i, hAlign) {
 if (this.offsets == null || i >= this.offsets.length) {
-if (hAlign == 4) return;
-this.offsets = JU.AU.ensureLengthI (this.offsets, i + 1);
-}this.offsets[i] = JV.JC.setHorizAlignment (this.offsets[i], hAlign);
+switch (hAlign) {
+case 0:
+case 4:
+return;
+}
+this.offsets = JU.AU.ensureLengthI (this.offsets, this.ac);
+}if (hAlign == 0) hAlign = 4;
+this.offsets[i] = JV.JC.setHorizAlignment (this.offsets[i], hAlign);
 var text = this.getLabel (i);
 if (text != null) text.setAlignment (hAlign);
 }, "~N,~N");
@@ -352,7 +380,7 @@ Clazz.defineMethod (c$, "setPointer",
  function (i, pointer) {
 if (this.offsets == null || i >= this.offsets.length) {
 if (pointer == 0) return;
-this.offsets = JU.AU.ensureLengthI (this.offsets, i + 1);
+this.offsets = JU.AU.ensureLengthI (this.offsets, this.ac);
 }this.offsets[i] = JV.JC.setPointer (this.offsets[i], pointer);
 var text = this.getLabel (i);
 if (text != null) text.pointer = pointer;
@@ -361,14 +389,14 @@ Clazz.defineMethod (c$, "setZPos",
  function (i, flag, TF) {
 if (this.offsets == null || i >= this.offsets.length) {
 if (!TF) return;
-this.offsets = JU.AU.ensureLengthI (this.offsets, i + 1);
+this.offsets = JU.AU.ensureLengthI (this.offsets, this.ac);
 }this.offsets[i] = JV.JC.setZPosition (this.offsets[i], TF ? flag : 0);
 }, "~N,~N,~B");
 Clazz.defineMethod (c$, "setFont", 
  function (i, fid) {
 if (this.fids == null || i >= this.fids.length) {
 if (fid == this.zeroFontId) return;
-this.fids = JU.AU.ensureLengthByte (this.fids, i + 1);
+this.fids = JU.AU.ensureLengthByte (this.fids, this.ac);
 }this.fids[i] = fid;
 this.bsFontSet.set (i);
 var text = this.getLabel (i);
