@@ -160,7 +160,7 @@ switch (this.tokAt (1)) {
 case 1073742001:
 if (this.listIsosurface (22)) return false;
 break;
-case 1073742102:
+case 134217762:
 var pt = 2;
 var type = (this.tokAt (pt) == 1073742138 ? "" : eval.optParameterAsString (pt));
 if (type.equalsIgnoreCase ("chemicalShift")) type = "cs";
@@ -517,7 +517,7 @@ break;
 case 1073742048:
 propertyName = "nohead";
 break;
-case 1073741861:
+case 1073741860:
 propertyName = "isbarb";
 break;
 case 1073742130:
@@ -892,7 +892,7 @@ var isIsosurface = (iShape == 24);
 var isPmesh = (iShape == 29);
 var isPlot3d = (iShape == 30);
 var isLcaoCartoon = (iShape == 26);
-var isSilent = (isLcaoCartoon || this.tokAt (1) == 12291);
+var isSilent = (isLcaoCartoon || this.tokAt (1) == 12291 || eval.$isStateScript);
 var surfaceObjectSeen = false;
 var planeSeen = false;
 var isMapped = false;
@@ -938,6 +938,7 @@ var idSeen = (this.initIsosurface (iShape) != null);
 var isWild = (idSeen && this.getShapeProperty (iShape, "ID") == null);
 var isColorSchemeTranslucent = false;
 var isInline = false;
+var isSign = false;
 var onlyOneModel = null;
 var translucency = null;
 var colorScheme = null;
@@ -1224,8 +1225,8 @@ break;
 case 1073742147:
 case 1765808134:
 idSeen = true;
-var isSign = (eval.theTok == 1073742147);
-if (isSign) {
+if (eval.theTok == 1073742147) {
+isSign = true;
 sbCommand.append (" sign");
 this.addShapeProperty (propertyList, "sign", Boolean.TRUE);
 } else {
@@ -1891,7 +1892,19 @@ case 4:
 var firstPass = (!surfaceObjectSeen && !planeSeen);
 propertyName = (firstPass && !isMapped ? "readFile" : "mapColor");
 var filename = this.paramAsStr (i);
-if (filename.startsWith ("=") && filename.length > 1) {
+var checkWithin = false;
+if (filename.startsWith ("*") && filename.length > 1) {
+if (filename.startsWith ("**")) {
+if (Float.isNaN (sigma)) this.addShapeProperty (propertyList, "sigma", Float.$valueOf (sigma = 3));
+if (!isSign) {
+isSign = true;
+sbCommand.append (" sign");
+this.addShapeProperty (propertyList, "sign", Boolean.TRUE);
+}}if (!Float.isNaN (sigma)) this.showString ("using sigma=" + sigma);
+filename = this.vwr.setLoadFormat (filename, '_', false);
+checkWithin = true;
+} else if (filename.startsWith ("=") && filename.length > 1) {
+checkWithin = true;
 var info = this.vwr.setLoadFormat (filename, '_', false);
 filename = info[0];
 var strCutoff = (!firstPass || !Float.isNaN (cutoff) ? null : info[1]);
@@ -1922,8 +1935,9 @@ sigma = NaN;
 this.addShapeProperty (propertyList, "sigma", Float.$valueOf (sigma));
 }this.addShapeProperty (propertyList, "cutoff", Float.$valueOf (cutoff));
 sbCommand.append (" cutoff ").appendF (cutoff);
-}}if (ptWithin == 0) {
-onlyOneModel = "=xxxx";
+}}}if (checkWithin) {
+if (ptWithin == 0) {
+onlyOneModel = filename;
 if (modelIndex < 0) modelIndex = this.vwr.am.cmi;
 bs = this.vwr.getModelUndeletedAtomsBitSet (modelIndex);
 if (bs.nextSetBit (0) >= 0) {
