@@ -41,8 +41,8 @@ Clazz.instantialize (this, arguments);
 }, JS, "ScriptTokenParser");
 Clazz.defineMethod (c$, "compileExpressions", 
 function () {
-var isScriptExpression = ((this.tokCommand == 134222853 || this.tokCommand == 4124) && this.tokAt (2) == 268435472);
-this.isEmbeddedExpression = (isScriptExpression || (this.tokCommand != 0 && (this.tokCommand != 134320137 && this.tokCommand != 102436 && this.tokCommand != 364558 && this.tokCommand != 102412 || this.tokenCommand.intValue != 2147483647) && this.tokCommand != 102409 && !JS.T.tokAttr (this.tokCommand, 12288) && (this.nTokens > 2 || !JS.T.tokAttr (this.tokCommand, 20480))));
+var isScriptExpression = ((this.tokCommand == 134222850 || this.tokCommand == 4124) && this.tokAt (2) == 268435472);
+this.isEmbeddedExpression = (isScriptExpression || (this.tokCommand != 0 && (this.tokCommand != 134320141 && this.tokCommand != 102436 && this.tokCommand != 364558 && this.tokCommand != 102412 || this.tokenCommand.intValue != 2147483647) && this.tokCommand != 102409 && !JS.T.tokAttr (this.tokCommand, 12288) && (this.nTokens > 2 || !JS.T.tokAttr (this.tokCommand, 20480))));
 this.isMathExpressionCommand = (this.tokCommand == 1073741824 || isScriptExpression || JS.T.tokAttr (this.tokCommand, 36864));
 var checkExpression = this.isEmbeddedExpression || (JS.T.tokAttr (this.tokCommand, 12288));
 if (this.tokAt (1) == 1073742330 && JS.T.tokAttr (this.tokCommand, 12288)) checkExpression = false;
@@ -55,7 +55,7 @@ if (this.tokAt (i) == 12290) nDefined++;
 size -= nDefined;
 if (this.isNewSet) {
 if (size == 1) {
-this.atokenInfix[0] = JS.T.tv (134320137, 0, this.atokenInfix[0].value);
+this.atokenInfix[0] = JS.T.tv (134320141, 0, this.atokenInfix[0].value);
 this.isNewSet = false;
 }}return ((this.isNewSet || this.isSetBrace) && size < this.ptNewSetModifier + 2 ? this.commandExpected () : size == 1 || !JS.T.tokAttr (this.tokCommand, 262144) ? true : this.error (0));
 });
@@ -107,7 +107,7 @@ for (var i = 0; i < firstToken && this.addNextToken (); i++) {
 while (this.moreTokens ()) {
 if (this.isEmbeddedExpression) {
 while (!this.isExpressionNext ()) {
-if (this.tokPeekIs (1073741824) && !(this.tokCommand == 134222851 && this.itokenInfix == 1)) {
+if (this.tokPeekIs (1073741824) && !(this.tokCommand == 134222849 && this.itokenInfix == 1)) {
 var name = this.atokenInfix[this.itokenInfix].value;
 var t = JS.T.getTokenFromName (name);
 if (t != null) if (!this.isMathExpressionCommand && this.lastToken.tok != 12290 || (this.lastToken.tok == 1073742336 || this.tokAt (this.itokenInfix + 1) == 268435472) && !this.isUserFunction (name)) {
@@ -268,16 +268,16 @@ this.itokenInfix++;
 return this.addTokenToPostfixToken (token);
 }, "~N,JS.T");
 Clazz.defineMethod (c$, "clauseOr", 
- function (allowComma) {
+ function (allowCommaAsOr) {
 this.haveString = false;
 if (!this.clauseAnd ()) return false;
 if (this.isEmbeddedExpression && this.lastToken.tok == 1073742326) return true;
 var tok;
-while ((tok = this.tokPeek ()) == 268435536 || tok == 268435537 || tok == 268435538 || allowComma && tok == 268435504) {
+while ((tok = this.tokPeek ()) == 268435536 || tok == 268435537 || tok == 268435538 || allowCommaAsOr && tok == 268435504) {
 if (tok == 268435504 && !this.haveString) this.addSubstituteTokenIf (268435504, JS.T.tokenOr);
  else this.addNextToken ();
 if (!this.clauseAnd ()) return false;
-if (allowComma && (this.lastToken.tok == 1073742338 || this.lastToken.tok == 10)) this.haveString = true;
+if (allowCommaAsOr && (this.lastToken.tok == 1073742338 || this.lastToken.tok == 10)) this.haveString = true;
 }
 return true;
 }, "~B");
@@ -310,7 +310,7 @@ return this.error (4);
 case 1073742327:
 case 10:
 case 268435632:
-case 136314891:
+case 136314895:
 case 2097160:
 case 2097159:
 case 2097162:
@@ -326,19 +326,19 @@ return this.addTokenToPostfixInt (1073742359, this.fixModelSpec (this.getToken (
 case 1094713349:
 case 1094713350:
 return this.clauseCell (tok);
-case 134217734:
-case 1275203591:
-return this.clauseConnected (tok == 1275203591);
-case 134218759:
-case 134218760:
+case 134217736:
+case 1275203608:
+return this.clauseConnected (tok == 1275203608);
+case 134218756:
+case 134218757:
 return this.clauseSubstructure ();
-case 134217749:
-case 134353929:
-return this.clauseWithin (tok == 134217749);
+case 134217759:
+case 134353926:
+return this.clauseWithin (tok == 134217759);
 case 12290:
 return this.clauseDefine (false, false);
 case 1677721602:
-case 1745489933:
+case 1745489939:
 this.addNextToken ();
 if (this.tokPeekIs (10)) this.addNextToken ();
  else if (this.tokPeekIs (12290)) return this.clauseDefine (false, false);
@@ -357,7 +357,7 @@ if (JS.T.tokAttr (tok, 1077936128)) {
 var itemp = this.itokenInfix;
 var isOK = this.clauseComparator (true);
 if (isOK || this.itokenInfix != itemp) return isOK;
-if (tok == 1237320710) {
+if (tok == 1237320707) {
 return this.clauseSubstructure ();
 }}return this.addNextToken ();
 }
@@ -385,7 +385,7 @@ while (nBrace != 0) {
 if (this.tokPeekIs (1073742332)) {
 if (this.isExpressionNext ()) {
 this.addTokenToPostfixToken (JS.T.o (1073742325, "implicitExpressionBegin"));
-if (!this.clauseOr (true)) return false;
+if (!this.clauseOr (false)) return false;
 if (this.lastToken !== JS.T.tokenCoordinateEnd) {
 this.addTokenToPostfixToken (JS.T.tokenExpressionEnd);
 }} else {
@@ -474,9 +474,9 @@ case 12290:
 break;
 case 1073741916:
 case 1073742128:
-case 134218759:
-case 134218760:
-case 1237320710:
+case 134218756:
+case 134218757:
+case 1237320707:
 case 1073741925:
 case 1073742189:
 this.addTokenToPostfix (4, this.theValue);
@@ -508,11 +508,11 @@ case 1086324740:
 case 1073742329:
 case 1086326789:
 case 1086324742:
-case 1613369372:
-case 136314891:
+case 1747587102:
+case 136314895:
 case 1094717454:
 case 1094713360:
-case 134217743:
+case 134217750:
 case 134219265:
 case 1094713362:
 case 1086324744:
@@ -532,7 +532,7 @@ if (key == null) this.addTokenToPostfix (3, Float.$valueOf (distance));
 var done = false;
 while (!done) {
 if (tok0 != 0 && !this.addNextTokenIf (268435504)) break;
-if (tok0 == 0) tok0 = 134353929;
+if (tok0 == 0) tok0 = 134353926;
 var isCoordOrPlane = false;
 tok = this.tokPeek ();
 if (isWithin) {
@@ -549,7 +549,7 @@ if (key == null) {
 switch (tok) {
 case 134219265:
 case 1073742329:
-case 134217743:
+case 134217750:
 isCoordOrPlane = true;
 this.addNextToken ();
 break;
@@ -561,7 +561,7 @@ done = true;
 break;
 case 1086324742:
 case 1648363544:
-case 1613369372:
+case 1747587102:
 this.getToken ();
 this.addTokenToPostfix (4, JS.T.nameOf (tok));
 break;
