@@ -521,9 +521,7 @@ return JU.PT.packageJSON (infoType, (s == null ? sb.toString () : s));
 c$.nonArrayString = Clazz.defineMethod (c$, "nonArrayString", 
 function (x) {
 {
-var s = x.constructor.toString();
-var pt = s.indexOf("Array(");
-return (pt <= 0 || pt >= s.indexOf("{") ? x.toString() : null);
+return (x.constructor == Array || x.BYTES_PER_ELEMENT ? null : x.toString());
 }}, "~O");
 c$.byteArrayToJSON = Clazz.defineMethod (c$, "byteArrayToJSON", 
 function (data) {
