@@ -5,6 +5,7 @@ this.ms = null;
 this.shapes = null;
 this.vwr = null;
 this.bsRenderableAtoms = null;
+this.bsSlabbedInternal = null;
 this.navMinMax = null;
 Clazz.instantialize (this, arguments);
 }, JV, "ShapeManager");
@@ -15,6 +16,7 @@ Clazz.makeConstructor (c$,
 function (vwr) {
 this.vwr = vwr;
 this.bsRenderableAtoms =  new JU.BS ();
+this.bsSlabbedInternal =  new JU.BS ();
 }, "JV.Viewer");
 Clazz.defineMethod (c$, "findNearestShapeAtomIndex", 
 function (x, y, closest, bsNot) {
@@ -228,33 +230,36 @@ if (shape != null) shape.setAtomClickability ();
 }
 });
 Clazz.defineMethod (c$, "finalizeAtoms", 
-function (checkAtoms, finalizeParams) {
+function (bsTranslateSelected, finalizeParams) {
 var vwr = this.vwr;
-if (finalizeParams) vwr.finalizeTransformParameters ();
 var tm = vwr.tm;
-var bs = this.bsRenderableAtoms;
-var bsAtoms = (checkAtoms ? tm.bsSelectedAtoms : null);
-if (bsAtoms != null) {
-var ptCenter = this.ms.getAtomSetCenter (bsAtoms);
+if (finalizeParams) vwr.finalizeTransformParameters ();
+if (bsTranslateSelected != null) {
+var ptCenter = this.ms.getAtomSetCenter (bsTranslateSelected);
 var pt =  new JU.P3 ();
 tm.transformPt3f (ptCenter, pt);
 pt.add (tm.ptOffset);
 tm.unTransformPoint (pt, pt);
 pt.sub (ptCenter);
-vwr.setAtomCoordsRelative (pt, bsAtoms);
+vwr.setAtomCoordsRelative (pt, bsTranslateSelected);
 tm.ptOffset.set (0, 0, 0);
 tm.bsSelectedAtoms = null;
-}this.ms.getAtomsInFrame (bs);
+}var bsOK = this.bsRenderableAtoms;
+this.ms.getAtomsInFrame (bsOK);
 var vibrationVectors = this.ms.vibrations;
 var vibs = (vibrationVectors != null && tm.vibrationOn);
 var checkOccupancy = (this.ms.bsModulated != null && this.ms.occupancies != null);
 var atoms = this.ms.at;
 var occ;
 var haveMods = false;
-for (var i = bs.nextSetBit (0); i >= 0; i = bs.nextSetBit (i + 1)) {
+var bsSlabbed = this.bsSlabbedInternal;
+bsSlabbed.clearAll ();
+for (var i = bsOK.nextSetBit (0); i >= 0; i = bsOK.nextSetBit (i + 1)) {
 var atom = atoms[i];
 var screen = (vibs && atom.hasVibration () ? tm.transformPtVib (atom, vibrationVectors[i]) : tm.transformPt (atom));
-atom.sX = screen.x;
+if (screen.z == 1 && tm.internalSlab && tm.xyzIsSlabbedInternal (atom)) {
+bsSlabbed.set (i);
+}atom.sX = screen.x;
 atom.sY = screen.y;
 atom.sZ = screen.z;
 var d = Math.abs (atom.madAtom);
@@ -281,30 +286,30 @@ for (var i = 0; i < moleculeCount; i++) {
 var m = molecules[i];
 var j = 0;
 var pt = m.firstAtomIndex;
-if (!bs.get (pt)) continue;
+if (!bsOK.get (pt)) continue;
 for (; j < m.ac; j++, pt++) if (gdata.isClippedZ (atoms[pt].sZ - (atoms[pt].sD >> 1))) break;
 
 if (j != m.ac) {
 pt = m.firstAtomIndex;
 for (var k = 0; k < m.ac; k++) {
-bs.clear (pt);
+bsOK.clear (pt);
 atoms[pt++].sZ = 0;
 }
 }}
-}for (var i = bs.nextSetBit (0); i >= 0; i = bs.nextSetBit (i + 1)) {
+}for (var i = bsOK.nextSetBit (0); i >= 0; i = bsOK.nextSetBit (i + 1)) {
 var atom = atoms[i];
 if (gdata.isClippedZ (atom.sZ - (slabByAtom ? atoms[i].sD >> 1 : 0))) {
 atom.setClickable (0);
 var r = Clazz.doubleToInt ((slabByAtom ? -1 : 1) * atom.sD / 2);
 if (atom.sZ + r < minZ || atom.sZ - r > maxZ || !gdata.isInDisplayRange (atom.sX, atom.sY)) {
-bs.clear (i);
+bsOK.clear (i);
 }}}
 }if (this.ms.ac == 0 || !vwr.getShowNavigationPoint ()) return null;
 var minX = 2147483647;
 var maxX = -2147483648;
 var minY = 2147483647;
 var maxY = -2147483648;
-for (var i = bs.nextSetBit (0); i >= 0; i = bs.nextSetBit (i + 1)) {
+for (var i = bsOK.nextSetBit (0); i >= 0; i = bsOK.nextSetBit (i + 1)) {
 var atom = atoms[i];
 if (atom.sX < minX) minX = atom.sX;
 if (atom.sX > maxX) maxX = atom.sX;
@@ -316,7 +321,7 @@ this.navMinMax[1] = maxX;
 this.navMinMax[2] = minY;
 this.navMinMax[3] = maxY;
 return this.navMinMax;
-}, "~B,~B");
+}, "JU.BS,~B");
 Clazz.defineMethod (c$, "setModelSet", 
 function (modelSet) {
 this.ms = this.vwr.ms = modelSet;
