@@ -19,7 +19,7 @@ b[2]+b[3]+"]");break;case 1073742080:b=NaN,this.isFloatParameter(++a)&&(b=this.f
 1275069441==a),this.showString(0>c?"cache cleared":c+" bytes "+(1275069441==a?" cached":" removed")));break;default:this.invArg()}});l(c$,"calculate",function(){var a=!1,c=!1,d=null,b=null,e=this.e,g=-2147483648;if(2<=(e.iToken=e.slen))switch(e.clearDefinedVariableAtomSets(),this.getToken(1).tok){case 1073741824:this.checkLength(2);break;case 1631586315:this.checkLength(2);if(this.chk)return;g=this.vwr.calculateFormalCharges(null);this.showString(J.i18n.GT.i(J.i18n.GT._("{0} charges modified"),g));
 return;case 1075838996:this.checkLength(2);this.chk||this.vwr.ms.assignAromaticBondsBs(!0,null);return;case 1613238294:2!=e.slen&&(d=(c=1639976963==this.tokAt(++e.iToken))?this.vwr.bsA():this.atomExpressionAt(e.iToken),!c&&1639976963!=this.tokAt(++e.iToken)&&(b=this.atomExpressionAt(e.iToken)));if(this.chk)return;g=this.vwr.autoHbond(d,b,!1);-2147483648!=g&&e.report(J.i18n.GT.i(J.i18n.GT._("{0} hydrogen bonds"),Math.abs(g)),!1);return;case 1612709900:(a=1073742335==this.tokAt(2))&&e.iToken++;d=this.slen==
 (a?3:2)?null:this.atomExpressionAt(a?3:2);e.checkLast(e.iToken);this.chk||(this.vwr.addHydrogens(d,!1,!1),a&&(null==d&&(d=this.vwr.bsA()),this.vwr.makeConnections(0.1,1E8,515,1073742025,d,d,null,!1,!1,0),this.vwr.ms.assignAromaticBondsBs(!0,null)));return;case 1111492619:e.iToken=1;d=2==this.slen?null:this.atomExpressionAt(2);e.checkLast(e.iToken);if(!this.chk)try{this.vwr.calculatePartialCharges(d)}catch(f){if(M(f,JV.JmolAsyncException))e.loadFileResourceAsync(f.getFileName());else throw f;}return;
-case 1088421903:case 1073742102:this.chk||(1275203608==e.tokAt(2)?this.showString(this.getShapePropertyIndex(21,"symmetry",1073742334==this.tokAt(3)?0:1)):this.showString(this.vwr.ms.calculatePointGroup(this.vwr.bsA())));return;case 1111490574:this.checkLength(2);this.chk||(this.vwr.calculateStraightness(),this.vwr.addStateScript("set quaternionFrame '"+this.vwr.getQuaternionFrame()+"'; calculate straightness",!1,!0));return;case 1639976963:d=4>this.slen?null:this.atomExpressionAt(2);switch(this.tokAt(++e.iToken)){case 4138:break;
+case 1088421903:case 134217762:this.chk||(1275203608==e.tokAt(2)?this.showString(this.getShapePropertyIndex(21,"symmetry",1073742334==this.tokAt(3)?0:1)):this.showString(this.vwr.ms.calculatePointGroup(this.vwr.bsA())));return;case 1111490574:this.checkLength(2);this.chk||(this.vwr.calculateStraightness(),this.vwr.addStateScript("set quaternionFrame '"+this.vwr.getQuaternionFrame()+"'; calculate straightness",!1,!0));return;case 1639976963:d=4>this.slen?null:this.atomExpressionAt(2);switch(this.tokAt(++e.iToken)){case 4138:break;
 case 1073741916:if(this.chk)return;e.showString(this.vwr.getAnnotationParser(!0).calculateDSSRStructure(this.vwr,d));return;case 1073741915:c=!0;break;case 0:c=this.vwr.getBoolean(603979826);break;default:this.invArg()}this.chk||this.showString(this.vwr.calculateStructures(d,c,!0));return;case 659482:d=e.iToken+1<this.slen?this.atomExpressionAt(++e.iToken):null;b=e.iToken+1<this.slen?this.atomExpressionAt(++e.iToken):null;this.checkLength(++e.iToken);this.chk||(g=this.vwr.calculateStruts(d,b),0<g&&
 (this.setShapeProperty(1,"type",Integer.$valueOf(32768)),e.setShapePropertyBs(1,"color",Integer.$valueOf(16777215),null),e.setShapeTranslucency(1,"","translucent",0.5,null),this.setShapeProperty(1,"type",Integer.$valueOf(1023))),this.showString(J.i18n.GT.i(J.i18n.GT._("{0} struts added"),g)));return;case 2097180:a=!0;case 1111490575:c=!1;switch(this.tokAt(2)){case 134217759:e.iToken++;break;case 0:c=!a;break;case 1073741952:c=!0;e.iToken++;break;default:c=!0}d=e.iToken+1<this.slen?this.atomExpressionAt(++e.iToken):
 this.vwr.bsA();this.checkLength(++e.iToken);this.chk||this.vwr.calculateSurface(d,c?3.4028235E38:-1);return}e.errorStr2(53,"CALCULATE","aromatic? hbonds? hydrogen? formalCharge? partialCharge? pointgroup? straightness? structure? struts? surfaceDistance FROM? surfaceDistance WITHIN?")});l(c$,"capture",function(){if(!this.chk&&!this.vwr.allowCapture())this.showString("Cannot capture on this platform");else{var a=this.vwr.captureParams,c=null==a?"GIF":a.get("type"),d=0,b=0,e=this.e.slen,g="",f=1073742070!=
@@ -97,13 +97,13 @@ this.vwr.getModelNumberDotted(j-1)+(0<v.length?" created: "+v+(s?e:""):""));retu
 this.invArg();s="unitCell";p=Boolean.TRUE;b=!0;break;case 1073742072:this.e.restrictSelected(!1,!1),a.theTok=1073742335;case 1073742335:case 12291:case 1073742334:(m+1!=this.slen||b||1<h)&&this.error(18);s=1073742334==a.theTok?"off":1073742335==a.theTok?"on":"delete";e=!0;break;case 2:null!=y&&this.invArg();s="nVertices";p=Integer.$valueOf(this.intParameter(m));b=!0;268435504==this.tokAt(m+1)&&m++;break;case 1677721602:null!=y&&this.invArg();0<h&&this.invPO();b=!0;s="bonds";c=!0;break;case 1665140738:m++;
 case 3:null!=y&&this.invArg();0<h&&this.invPO();s="radius";p=Float.$valueOf(this.floatParameter(m));b=!0;break;case 1073742066:if(!this.isFloatParameter(m+1)){k=this.getPoint3f(++m,!0);m=a.iToken;continue}case 1073741937:this.setShapeProperty(21,"collapsed",Boolean.TRUE);case 1073742099:case 1073741924:s=JS.T.nameOf(a.theTok);switch(this.tokAt(m+1)){case 268435860:case 268435504:m++}p=Float.$valueOf(this.floatParameter(++m));break;case 1094717454:null==y&&this.invArg();s="model";p=Integer.$valueOf(this.intParameter(++m));
 break;case 1073742170:(1<h||null!=y&&!d)&&this.invPO();h=3;10==this.tokAt(++m)||1073742325==this.tokAt(m)?(s=b?"to":"toBitSet",p=this.atomExpressionAt(m)):a.isArrayParameter(m)?(s="toVertices",p=a.getPointArray(m,-1,!1)):this.error(19);m=a.iToken;b=!0;break;case 10:case 1073742325:g&&this.invPO();switch(++h){case 1:null!=y&&this.invArg();s="centers";break;case 2:s="to";b=!0;break;default:a.bad()}p=this.atomExpressionAt(m);m=a.iToken;b=(new Boolean(b|m+1==this.slen)).valueOf();break;case 1765808134:case 603979967:case 1073742074:l=
-this.getColorTrans(a,m,!0,n);m=a.iToken;continue;case 1073741886:g&&this.error(18);g=!0;this.isFloatParameter(m+1)&&this.setShapeProperty(21,"faceCenterOffset",Float.$valueOf(this.floatParameter(++m)));s="collapsed";p=Boolean.TRUE;break;case 1073742044:case 1073741933:case 1073741956:f&&this.error(18);f=!0;s=JS.T.nameOf(a.theTok);break;case 1073742182:case 1073742060:case 1073741862:case 1073741958:case 1073741964:continue;case 1073741974:case 268435633:case 1073741824:case 4:if(!a.isColorParam(m)){1!=
+this.getColorTrans(a,m,!0,n);m=a.iToken;continue;case 1073741886:g&&this.error(18);g=!0;this.isFloatParameter(m+1)&&this.setShapeProperty(21,"faceCenterOffset",Float.$valueOf(this.floatParameter(++m)));s="collapsed";p=Boolean.TRUE;break;case 1073742044:case 1073741933:case 1073741956:f&&this.error(18);f=!0;s=JS.T.nameOf(a.theTok);break;case 1073742182:case 1073742060:case 1073741861:case 1073741958:case 1073741964:continue;case 1073741974:case 268435633:case 1073741824:case 4:if(!a.isColorParam(m)){1!=
 m&&this.invPO();y=1073741974==a.theTok?this.stringParameter(++m):a.optParameterAsString(m);this.setShapeProperty(21,"thisID",y);this.setShapeProperty(21,"model",Integer.$valueOf(this.vwr.am.cmi));if(!a.isCenterParameter(m+1))continue;s="center";p=this.centerParameter(++m);m=a.iToken;d=!0;break}default:if(a.isColorParam(m)){n[0]=a.getArgbParam(m);m=a.iToken;continue}this.invArg()}null!=s&&this.setShapeProperty(21,s,p);if(e)return}b?(!g&&c&&this.setShapeProperty(21,"bonds",null),this.setShapeProperty(21,
 "generate",null)):!f&&(null==k&&Float.isNaN(j))&&this.error(19);null!=k&&this.setShapeProperty(21,"offset",k);Float.isNaN(j)||this.setShapeProperty(21,"scale",Float.$valueOf(j));-2147483648!=n[0]&&this.setShapeProperty(21,"colorThis",Integer.$valueOf(n[0]));3.4028235E38!=l&&a.setShapeTranslucency(21,"","translucentThis",l,null);this.setShapeProperty(21,"init",Boolean.FALSE)});l(c$,"write",function(a){var c=this.e,d=1,b=1,e=null,g=null,f=null,j=null,h="SPT",g=!0,l=!1;null==a?(a=this.st,l=this.vwr.isApplet&&
 !this.vwr.isSignedApplet||!this.vwr.haveAccess(JV.Viewer.ACCESS.ALL)||0<this.vwr.fm.getPathForAllFiles().length):(d=b=0,g=!1,l=!g);var n=JS.CmdExt.tokAtArray(d,a);if(4==n&&!g){var k=JS.T.getTokenFromName(JS.SV.sValue(a[0]).toLowerCase());null!=k&&(n=k.tok)}switch(n){case 0:break;case 134221850:case 4138:case 1715472409:return g=this.plot(a),l?g:this.writeMsg(g);case 134222850:c.isArrayParameter(d+1)&&(e=c.stringParameterSet(++d),j=f=".",b=d=c.iToken+1,n=this.tokAt(d));break;default:h=JS.SV.sValue(this.tokenAt(d,
-a)).toUpperCase()}var y=this.vwr.getExportDriverList(),k=null,m=g?this.slen:a.length,s="",p=null,B=null,v=0,x="PNGJ",w=!1,r=null,z=-1,t=-1,q=!1,u=null;0!=n&&(g&&1<this.slen&&1073741848==this.tokAt(this.slen-2))&&(h=this.paramAsStr(this.slen-1).toUpperCase(),b=m,m-=2,n=0);switch(n){case 15:case 6:h="VAR";B=this.tokenAt(d++,a);break;case 1073741983:h="INLINE";k=JS.SV.sValue(this.tokenAt(++d,a));d++;break;case 1073742102:h="PGRP";d++;s=JS.SV.sValue(this.tokenAt(d,a)).toLowerCase();s.equals("draw")&&
-d++;break;case 1073742329:d++;w=!0;break;case 1073742158:case 134222850:for(p=JS.SV.sValue(this.tokenAt(++d,a)).toLowerCase();p.equals("localpath")||p.equals("remotepath");)p.equals("localpath")?f=JS.SV.sValue(this.tokenAt(++d,a)):j=JS.SV.sValue(this.tokenAt(++d,a)),p=JS.SV.sValue(this.tokenAt(++d,a)).toLowerCase();h="SPT";break;case 1228935687:case 134320141:case 1610616855:case 135180:case 1073742015:case 1073742018:case 1073877011:case 1073877010:case 135188:d++;break;case 1073741991:h="ZIPALL";
-d++;break;case 36868:h="VAR";d+=2;break;case 4115:case 4120:case 1073742139:case 4166:case 1073741824:case 4:switch(n){case 4115:var C;d+1<m&&1073742325==a[++d].tok||10==a[d].tok?(C=c.atomExpression(a,d,0,!0,!1,null,!0),d=c.iToken+1):C=this.vwr.getAllAtoms();this.chk||(r=this.vwr.ms.getModelBS(C,!0));break;case 4120:h="IMAGE";d++;break;case 1073742139:p=JS.SV.sValue(this.tokenAt(++d,a)).toUpperCase();JU.PT.isOneOf(p,";PNG;PNGJ;")&&(x=p,d++);break;case 4166:v=c.intParameterRange(++d,1,10);if(2147483647==
+a)).toUpperCase()}var y=this.vwr.getExportDriverList(),k=null,m=g?this.slen:a.length,s="",p=null,B=null,v=0,x="PNGJ",w=!1,r=null,z=-1,t=-1,q=!1,u=null;0!=n&&(g&&1<this.slen&&1073741848==this.tokAt(this.slen-2))&&(h=this.paramAsStr(this.slen-1).toUpperCase(),b=m,m-=2,n=0);switch(n){case 15:case 6:h="VAR";B=this.tokenAt(d++,a);break;case 1073741983:h="INLINE";k=JS.SV.sValue(this.tokenAt(++d,a));d++;break;case 134217762:h="PGRP";d++;s=JS.SV.sValue(this.tokenAt(d,a)).toLowerCase();s.equals("draw")&&d++;
+break;case 1073742329:d++;w=!0;break;case 1073742158:case 134222850:for(p=JS.SV.sValue(this.tokenAt(++d,a)).toLowerCase();p.equals("localpath")||p.equals("remotepath");)p.equals("localpath")?f=JS.SV.sValue(this.tokenAt(++d,a)):j=JS.SV.sValue(this.tokenAt(++d,a)),p=JS.SV.sValue(this.tokenAt(++d,a)).toLowerCase();h="SPT";break;case 1228935687:case 134320141:case 1610616855:case 135180:case 1073742015:case 1073742018:case 1073877011:case 1073877010:case 135188:d++;break;case 1073741991:h="ZIPALL";d++;
+break;case 36868:h="VAR";d+=2;break;case 4115:case 4120:case 1073742139:case 4166:case 1073741824:case 4:switch(n){case 4115:var C;d+1<m&&1073742325==a[++d].tok||10==a[d].tok?(C=c.atomExpression(a,d,0,!0,!1,null,!0),d=c.iToken+1):C=this.vwr.getAllAtoms();this.chk||(r=this.vwr.ms.getModelBS(C,!0));break;case 4120:h="IMAGE";d++;break;case 1073742139:p=JS.SV.sValue(this.tokenAt(++d,a)).toUpperCase();JU.PT.isOneOf(p,";PNG;PNGJ;")&&(x=p,d++);break;case 4166:v=c.intParameterRange(++d,1,10);if(2147483647==
 v)return"";this.chk||(this.vwr.tm.setVibrationPeriod(0),c.isJS||c.delayScript(100));d++;break;default:n=4120}if(4120==n&&d<a.length)if(n=JS.T.getTokenFromName(JS.SV.sValue(a[d]).toLowerCase()),null!=n&&(h=JS.SV.sValue(n).toUpperCase()),JU.PT.isOneOf(h,y.toUpperCase())){d++;h=h.substring(0,1).toUpperCase()+h.substring(1).toLowerCase();q=!0;g&&(u="Jmol."+h.toLowerCase());break}else if(JU.PT.isOneOf(h,";ZIP;ZIPALL;SPT;STATE;")){d++;break}else h="IMAGE";2==JS.CmdExt.tokAtArray(d,a)&&(z=JS.SV.iValue(this.tokenAt(d++,
 a)),t=JS.SV.iValue(this.tokenAt(d++,a)))}n=-2147483648;if(b<m){p=JS.SV.sValue(this.tokenAt(d,a));if(p.equalsIgnoreCase("clipboard")){if(this.chk)return""}else JU.PT.isOneOf(p.toLowerCase(),";jpg;jpeg;jpg64;jpeg64;gif;gift;pdf;ppm;png;pngj;pngt;")?(2==JS.CmdExt.tokAtArray(d+1,a)&&2==JS.CmdExt.tokAtArray(d+2,a)&&(z=JS.SV.iValue(this.tokenAt(++d,a)),t=JS.SV.iValue(this.tokenAt(++d,a))),2==JS.CmdExt.tokAtArray(d+1,a)&&(n=JS.SV.iValue(this.tokenAt(++d,a)))):JU.PT.isOneOf(p.toLowerCase(),";xyz;xyzrn;xyzvib;mol;sdf;v2000;v3000;json;pdb;pqr;cml;")&&
 (h=p.toUpperCase(),d+1==m&&d++);h.equals("IMAGE")&&JU.PT.isOneOf(p.toLowerCase(),";jpg;jpeg;jpg64;jpeg64;gif;gift;pdf;ppm;png;pngj;pngt;scene;")&&(h=p.toUpperCase(),d++)}d+2==m&&(b=JS.SV.sValue(this.tokenAt(++d,a)),0<b.length&&"."!=b.charAt(0)&&(h=p.toUpperCase()));switch(JS.CmdExt.tokAtArray(d,a)){case 0:l=!0;break;case 1073741884:break;case 805306401:u=h.equals("IMAGE")?"?jmol.png":"?jmol."+h.toLowerCase();break;case 1073741824:case 4:u=JS.SV.sValue(this.tokenAt(d,a));if(u.equalsIgnoreCase("clipboard")||
@@ -136,7 +136,7 @@ this.paramAsStr(2);if(e.startsWith("/")&&3==(g=this.slen)){if(!this.chk){e=e.sub
 '"'))));null!=f&&(b=""+f.get("spaceGroupInfo")+f.get("symmetryInfo"));break;case 1073742330:g=3;b=a.setObjectProperty();break;case 1678381065:this.chk||(b=this.vwr.ms.getBoundBoxCommand(!0));break;case 12289:this.chk||(b="center "+JU.Escape.eP(this.vwr.tm.fixedRotationCenter));break;case 135176:this.chk||(b=this.getShapeProperty(22,"command"));break;case 1228935687:this.chk||this.vwr.sm.clearConsole();if(2==this.slen){this.chk||(b=this.vwr.getCurrentFileAsString("script"));null==b&&(b="<unavailable>");
 break}g=3;c=this.paramAsStr(2);this.chk||(b=this.vwr.getFileAsString3(c,!0,null));break;case 4115:b=1073742327==this.tokAt(2)&&0<(g=3)?this.vwr.getModelFileInfoAll():this.vwr.getModelFileInfo();break;case 1610616855:f=2==(g=this.slen)?2147483647:this.intParameter(2);1>f&&this.invArg();this.chk||(this.vwr.sm.clearConsole(),0==a.scriptLevel&&this.vwr.removeCommand(),b=this.vwr.getSetHistory(f));break;case 135180:this.chk||(b=this.getShapeProperty(24,"jvxlDataXml"));break;case 1073877011:case 1073877010:a.optParameterAsString(2).equalsIgnoreCase("list")?
 (this.e.sm.loadShape(27),b=this.chk?"":this.getShapeProperty(27,"list -1"),g=3):(e=2==(g=this.slen)?-2147483648:this.intParameter(2),this.chk||(b=this.getMoJvxl(e,1073877011==f)));break;case 1094717454:this.chk||(b=this.vwr.ms.getModelInfoAsString());break;case 537006096:this.chk||(b=this.vwr.getMeasurementInfoAsString());break;case 1073741863:g=3;!this.chk&&this.slen==g&&(b=this.vwr.getOrientationText(this.tokAt(2),null));break;case 1073742132:f=this.tokAt(2),0==f?f=1073742132:g=3;case 1073742178:case 4129:this.chk||
-(b=this.vwr.getOrientationText(f,null));break;case 1073742077:g=2;if(3<this.slen)break;switch(f=this.tokAt(2)){case 1073742178:case 1073742132:case 4129:case 0:this.chk||(b=this.vwr.getOrientationText(f,null));break;default:e=a.optParameterAsString(2),b=this.vwr.getOrientationText(1073742035,e)}g=this.slen;break;case 1073742088:this.chk||(b=this.vwr.ms.getPDBHeader(this.vwr.am.cmi));break;case 1073742102:a=a.optParameterAsString(2);0==a.length&&(a=null);this.chk||this.showString(this.vwr.ms.getPointGroupAsString(this.vwr.bsA(),
+(b=this.vwr.getOrientationText(f,null));break;case 1073742077:g=2;if(3<this.slen)break;switch(f=this.tokAt(2)){case 1073742178:case 1073742132:case 4129:case 0:this.chk||(b=this.vwr.getOrientationText(f,null));break;default:e=a.optParameterAsString(2),b=this.vwr.getOrientationText(1073742035,e)}g=this.slen;break;case 1073742088:this.chk||(b=this.vwr.ms.getPDBHeader(this.vwr.am.cmi));break;case 134217762:a=a.optParameterAsString(2);0==a.length&&(a=null);this.chk||this.showString(this.vwr.ms.getPointGroupAsString(this.vwr.bsA(),
 !1,"show:"+a,0,0));return;case 1088421903:this.chk||(b=this.vwr.ms.getSymmetryInfoAsString());break;case 1073742176:this.chk||(b="transform:\n"+this.vwr.tm.matrixRotate.toString());break;case 4168:b="zoom "+(this.vwr.tm.zoomEnabled?""+this.vwr.tm.getZoomSetting():"off");break;case 1611272202:b=this.vwr.getShowFrank()?"frank ON":"frank OFF";break;case 1665140738:d="solventProbeRadius";break;case 1086324744:if(3==(g=this.slen)&&1073742334==this.tokAt(2))f=1086324743;case 1073741864:case 1086324740:case 1073742120:case 1113589787:case 1086324742:case 1140850689:case 1073741982:this.chk||
 (b=this.vwr.getChimeInfo(f));break;case 537022465:case 1610612738:case 20482:case 1612709912:c="?";break;case 1073742031:f=2==(g=this.slen)?null:this.paramAsStr(2);this.chk||(b=this.vwr.getBindingInfo(f));break;case 1073742015:this.chk||(c=this.vwr.getMenu(""));break;case 1073741824:d.equalsIgnoreCase("fileHeader")&&(this.chk||(b=this.vwr.ms.getPDBHeader(this.vwr.am.cmi)));break;case 1073741992:case 36868:d=this.paramAsStr(g++),e=a.getParameter(d,1073742190,!0),this.chk||(b=1073741992==f?e.toJSON():
 e.escape())}this.checkLength(g);this.chk||(null!=b?this.showString(b):null!=c?this.showString(d+" = "+c):null!=d&&(0<=d.indexOf(" ")?this.showString(d):this.showString(d+" = "+a.getParameter(d,1073742190,!0).escape())))}});l(c$,"stereo",function(){for(var a=J.c.STER.DOUBLE,c=-5,d=!1,b=null,e=0,g=1;g<this.slen;++g)if(this.e.isColorParam(g))1<e&&this.e.bad(),0==e&&(b=D(2,0)),d||(c=3),b[e]=this.e.getArgbParam(g),0==e++&&(b[1]=~b[0]),g=this.e.iToken;else switch(this.getToken(g).tok){case 1073742335:this.e.checkLast(this.e.iToken=
