@@ -42,6 +42,7 @@
 ,Clazz_getInheritedLevel
 ,Clazz_getParamsType
 ,Clazz_isAF
+,Clazz_isAB
 ,Clazz_isAI
 ,Clazz_isAS
 ,Clazz_isASS
@@ -793,6 +794,8 @@ this.defaultFontId = this.zeroFontId = this.vwr.gdata.getFont3DFSS ("SansSerif",
 this.defaultColix = 0;
 this.defaultBgcolix = 0;
 this.defaultOffset = JV.JC.LABEL_DEFAULT_OFFSET;
+this.defaultAlignment = 4;
+this.defaultPointer = 0;
 this.defaultZPos = 0;
 this.translucentAllowed = false;
 });
@@ -805,9 +808,11 @@ return;
 }if ("color" === propertyName) {
 var pid = J.c.PAL.pidOf (value);
 var colix = JU.C.getColixO (value);
-if (!this.setDefaults) for (var i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) this.setLabelColix (i, colix, pid);
+if (!this.setDefaults) {
+var n = this.checkColixLength (colix, bsSelected.length ());
+for (var i = bsSelected.nextSetBit (0); i >= 0 && i < n; i = bsSelected.nextSetBit (i + 1)) this.setLabelColix (i, colix, pid);
 
-if (this.setDefaults || !this.defaultsOnlyForNone) {
+}if (this.setDefaults || !this.defaultsOnlyForNone) {
 this.defaultColix = colix;
 this.defaultPaletteID = pid;
 }return;
@@ -815,8 +820,8 @@ this.defaultPaletteID = pid;
 if (this.strings == null) return;
 var val = (value).floatValue ();
 var scalePixelsPerMicron = (val == 0 ? 0 : 10000 / val);
-for (var i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) {
-if (this.strings.length <= i) continue;
+var n = Math.min (this.ac, this.strings.length);
+for (var i = bsSelected.nextSetBit (0); i >= 0 && i < n; i = bsSelected.nextSetBit (i + 1)) {
 var text = this.getLabel (i);
 if (text == null) {
 text = JM.Text.newLabel (this.vwr, null, this.strings[i], 0, 0, 0, scalePixelsPerMicron);
@@ -828,14 +833,17 @@ return;
 }if ("label" === propertyName) {
 this.setScaling ();
 var tokens = null;
+var nbs = this.checkStringLength (bsSelected.length ());
+if (this.defaultColix != 0 || this.defaultPaletteID != 0) this.checkColixLength (this.defaultColix, bsSelected.length ());
+if (this.defaultBgcolix != 0) this.checkBgColixLength (this.defaultBgcolix, bsSelected.length ());
 if (Clazz_instanceOf (value, JU.Lst)) {
 var list = value;
 var n = list.size ();
 tokens =  Clazz_newArray (-1, [null]);
-for (var pt = 0, i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) {
+for (var pt = 0, i = bsSelected.nextSetBit (0); i >= 0 && i < nbs; i = bsSelected.nextSetBit (i + 1)) {
 if (pt >= n) {
 this.setLabel (J.shape.Labels.nullToken, "", i, true);
-return;
+continue;
 }tokens[0] = null;
 this.setLabel (tokens, JS.SV.sValue (list.get (pt++)), i, true);
 }
@@ -845,6 +853,11 @@ tokens = (strLabel == null || strLabel.length == 0 ? J.shape.Labels.nullToken :
 for (var i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) this.setLabel (tokens, strLabel, i, true);
 
 }return;
+}if (propertyName.startsWith ("label:")) {
+this.setScaling ();
+this.checkStringLength (this.ac);
+this.setLabel ( Clazz_newArray (-1, [null]), propertyName.substring (6), (value).intValue (), false);
+return;
 }if ("clearBoxes" === propertyName) {
 this.labelBoxes = null;
 return;
@@ -852,13 +865,15 @@ return;
 return;
 }if ("bgcolor" === propertyName) {
 this.isActive = true;
-if (this.bsBgColixSet == null) this.bsBgColixSet =  new JU.BS ();
+if (this.bsBgColixSet == null) this.bsBgColixSet = JU.BS.newN (this.ac);
 var bgcolix = JU.C.getColixO (value);
-if (!this.setDefaults) for (var i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) this.setBgcolix (i, bgcolix);
+if (!this.setDefaults) {
+var n = this.checkBgColixLength (bgcolix, bsSelected.length ());
+for (var i = bsSelected.nextSetBit (0); i >= 0 && i < n; i = bsSelected.nextSetBit (i + 1)) this.setBgcolix (i, bgcolix);
 
-if (this.setDefaults || !this.defaultsOnlyForNone) this.defaultBgcolix = bgcolix;
+}if (this.setDefaults || !this.defaultsOnlyForNone) this.defaultBgcolix = bgcolix;
 return;
-}if (this.bsFontSet == null) this.bsFontSet =  new JU.BS ();
+}if (this.bsFontSet == null) this.bsFontSet = JU.BS.newN (this.ac);
 if ("fontsize" === propertyName) {
 var fontsize = (value).intValue ();
 if (fontsize < 0) {
@@ -877,9 +892,10 @@ if (this.setDefaults || !this.defaultsOnlyForNone) this.defaultFontId = fid;
 return;
 }if ("offset" === propertyName) {
 if (!(Clazz_instanceOf (value, Integer))) {
-if (!this.setDefaults) for (var i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) this.setPymolOffset (i, value);
+if (!this.setDefaults) {
+for (var i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) this.setPymolOffset (i, value);
 
-return;
+}return;
 }var offset = (value).intValue ();
 if (!this.setDefaults) for (var i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) this.setOffsets (i, offset);
 
@@ -887,9 +903,7 @@ if (this.setDefaults || !this.defaultsOnlyForNone) this.defaultOffset = offset;
 return;
 }if ("align" === propertyName) {
 var type = value;
-var hAlignment = 4;
-if (type.equalsIgnoreCase ("right")) hAlignment = 12;
- else if (type.equalsIgnoreCase ("center")) hAlignment = 8;
+var hAlignment = (type.equalsIgnoreCase ("right") ? 12 : type.equalsIgnoreCase ("center") ? 8 : 4);
 for (var i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) this.setHorizAlignment (i, hAlignment);
 
 if (this.setDefaults || !this.defaultsOnlyForNone) this.defaultAlignment = hAlignment;
@@ -921,14 +935,16 @@ var strLabelUNK = null;
 var tokensUNK = null;
 var strLabel;
 var tokens;
+var nstr = this.checkStringLength (bsSelected.length ());
+var bgcolix = this.defaultBgcolix;
+var nbg = this.checkBgColixLength (bgcolix, bsSelected.length ());
+var thisMad = (mode >= 0 ? 1 : -1);
 for (var i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) {
 var atom = this.atoms[i];
-if (this.formats == null || i >= this.formats.length) this.formats = JU.AU.ensureLengthS (this.formats, i + 1);
-if (this.strings != null && this.strings.length > i && this.strings[i] != null) {
-this.mads[i] = (mode == 0 && this.mads[i] < 0 || mode == 1 ? 1 : -1);
+if (i < nstr && this.strings[i] != null) {
+this.mads[i] = (mode == 1 || mode == 0 && this.mads[i] < 0 ? 1 : -1);
 } else {
-if (this.bsSizeSet == null) this.bsSizeSet =  new JU.BS ();
-this.strings = JU.AU.ensureLengthS (this.strings, i + 1);
+this.mads[i] = thisMad;
 if (atom.getGroup3 (false).equals ("UNK")) {
 if (strLabelUNK == null) {
 strLabelUNK = this.vwr.getStandardLabelFormat (1);
@@ -944,20 +960,12 @@ tokens = tokensPDB;
 }this.strings[i] = JM.LabelToken.formatLabelAtomArray (this.vwr, atom, tokens, '\0', null, this.ptTemp);
 this.formats[i] = strLabel;
 this.bsSizeSet.set (i);
-if ((this.bsBgColixSet == null || !this.bsBgColixSet.get (i)) && this.defaultBgcolix != 0) this.setBgcolix (i, this.defaultBgcolix);
-this.mads[i] = (mode >= 0 ? 1 : -1);
+if (i < nbg && !this.bsBgColixSet.get (i)) this.setBgcolix (i, this.defaultBgcolix);
 }this.setShapeVisibility (atom, this.strings != null && i < this.strings.length && this.strings[i] != null && this.mads[i] >= 0);
 }
 return;
-}if (propertyName.startsWith ("label:")) {
-this.setScaling ();
-this.setLabel ( new Array (1), propertyName.substring (6), (value).intValue (), false);
-return;
-}if ("textLabels" === propertyName) {
-this.setScaling ();
-var labels = value;
-for (var i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) this.setTextLabel (i, labels.get (Integer.$valueOf (i)), null);
-
+}if ("pymolLabels" === propertyName) {
+this.setPymolLabels (value, bsSelected);
 return;
 }if (propertyName === "deleteModelAtoms") {
 this.labelBoxes = null;
@@ -972,26 +980,48 @@ JU.BSUtil.deleteBits (this.bsFontSet, bsSelected);
 JU.BSUtil.deleteBits (this.bsBgColixSet, bsSelected);
 }this.setPropAS (propertyName, value, bsSelected);
 }, "~S,~O,JU.BS");
+Clazz_defineMethod (c$, "checkStringLength", 
+ function (n) {
+n = Math.min (this.ac, n);
+if (this.strings == null || n > this.strings.length) {
+this.formats = JU.AU.ensureLengthS (this.formats, n);
+this.strings = JU.AU.ensureLengthS (this.strings, n);
+if (this.bsSizeSet == null) this.bsSizeSet = JU.BS.newN (n);
+}return n;
+}, "~N");
+Clazz_defineMethod (c$, "checkBgColixLength", 
+ function (colix, n) {
+n = Math.min (this.ac, n);
+if (colix == 0) return (this.bgcolixes == null ? 0 : this.bgcolixes.length);
+if (this.bgcolixes == null || n > this.bgcolixes.length) this.bgcolixes = JU.AU.ensureLengthShort (this.bgcolixes, n);
+return n;
+}, "~N,~N");
+Clazz_defineMethod (c$, "setPymolLabels", 
+ function (labels, bsSelected) {
+this.setScaling ();
+var n = this.checkStringLength (this.ac);
+for (var i = bsSelected.nextSetBit (0); i >= 0 && i < n; i = bsSelected.nextSetBit (i + 1)) this.setPymolLabel (i, labels.get (Integer.$valueOf (i)), null);
+
+}, "java.util.Map,JU.BS");
 Clazz_defineMethod (c$, "setPymolOffset", 
  function (i, value) {
 var text = this.getLabel (i);
 if (text == null) {
-if (this.strings == null || this.strings.length <= i || this.strings[i] == null) return;
+if (this.strings == null || i >= this.strings.length || this.strings[i] == null) return;
 var fid = (this.bsFontSet != null && this.bsFontSet.get (i) ? this.fids[i] : -1);
 if (fid < 0) this.setFont (i, fid = this.defaultFontId);
 text = JM.Text.newLabel (this.vwr, javajs.awt.Font.getFont3D (fid), this.strings[i], this.getColix2 (i, this.atoms[i], false), this.getColix2 (i, this.atoms[i], true), 0, this.scalePixelsPerMicron);
-this.setTextLabel (i, text, this.formats[i]);
-if (text == null) return;
+this.setPymolLabel (i, text, this.formats[i]);
 }text.pymolOffset = value;
 }, "~N,~A");
 Clazz_defineMethod (c$, "setScaling", 
  function () {
 this.isActive = true;
-if (this.bsSizeSet == null) this.bsSizeSet =  new JU.BS ();
+if (this.bsSizeSet == null) this.bsSizeSet = JU.BS.newN (this.ac);
 this.isScaled = this.vwr.getBoolean (603979845);
 this.scalePixelsPerMicron = (this.isScaled ? this.vwr.getScalePixelsPerAngstrom (false) * 10000 : 0);
 });
-Clazz_defineMethod (c$, "setTextLabel", 
+Clazz_defineMethod (c$, "setPymolLabel", 
  function (i, t, format) {
 if (t == null) return;
 var label = t.getText ();
@@ -1030,8 +1060,6 @@ if (this.defaultFontId != this.zeroFontId) this.setFont (i, this.defaultFontId);
 Clazz_defineMethod (c$, "addString", 
  function (atom, i, label, strLabel) {
 this.setShapeVisibility (atom, label != null);
-if (this.strings == null || i >= this.strings.length) this.strings = JU.AU.ensureLengthS (this.strings, i + 1);
-if (this.formats == null || i >= this.formats.length) this.formats = JU.AU.ensureLengthS (this.formats, i + 1);
 var notNull = (strLabel != null);
 var isNew = (this.strings[i] == null);
 this.strings[i] = label;
@@ -1074,10 +1102,7 @@ if (this.colixes != null && ((text = this.getLabel (i)) != null)) text.colix = t
 }, "~N,~N,~N");
 Clazz_defineMethod (c$, "setBgcolix", 
  function (i, bgcolix) {
-if (this.bgcolixes == null || i >= this.bgcolixes.length) {
-if (bgcolix == 0) return;
-this.bgcolixes = JU.AU.ensureLengthShort (this.bgcolixes, i + 1);
-}this.bgcolixes[i] = bgcolix;
+this.bgcolixes[i] = bgcolix;
 this.bsBgColixSet.setBitTo (i, bgcolix != 0);
 var text = this.getLabel (i);
 if (text != null) text.bgcolix = bgcolix;
@@ -1085,19 +1110,23 @@ if (text != null) text.bgcolix = bgcolix;
 Clazz_defineMethod (c$, "setOffsets", 
  function (i, offset) {
 if (this.offsets == null || i >= this.offsets.length) {
-if (offset == 0) return;
-this.offsets = JU.AU.ensureLengthI (this.offsets, i + 1);
-}if (offset == 0) offset = JV.JC.LABEL_DEFAULT_OFFSET;
-this.offsets[i] = (this.offsets[i] & 63) | offset;
+if (offset == JV.JC.LABEL_DEFAULT_OFFSET) return;
+this.offsets = JU.AU.ensureLengthI (this.offsets, this.ac);
+}this.offsets[i] = (this.offsets[i] & 63) | offset;
 var text = this.getLabel (i);
 if (text != null) text.setOffset (offset);
 }, "~N,~N");
 Clazz_defineMethod (c$, "setHorizAlignment", 
  function (i, hAlign) {
 if (this.offsets == null || i >= this.offsets.length) {
-if (hAlign == 4) return;
-this.offsets = JU.AU.ensureLengthI (this.offsets, i + 1);
-}this.offsets[i] = JV.JC.setHorizAlignment (this.offsets[i], hAlign);
+switch (hAlign) {
+case 0:
+case 4:
+return;
+}
+this.offsets = JU.AU.ensureLengthI (this.offsets, this.ac);
+}if (hAlign == 0) hAlign = 4;
+this.offsets[i] = JV.JC.setHorizAlignment (this.offsets[i], hAlign);
 var text = this.getLabel (i);
 if (text != null) text.setAlignment (hAlign);
 }, "~N,~N");
@@ -1105,7 +1134,7 @@ Clazz_defineMethod (c$, "setPointer",
  function (i, pointer) {
 if (this.offsets == null || i >= this.offsets.length) {
 if (pointer == 0) return;
-this.offsets = JU.AU.ensureLengthI (this.offsets, i + 1);
+this.offsets = JU.AU.ensureLengthI (this.offsets, this.ac);
 }this.offsets[i] = JV.JC.setPointer (this.offsets[i], pointer);
 var text = this.getLabel (i);
 if (text != null) text.pointer = pointer;
@@ -1114,14 +1143,14 @@ Clazz_defineMethod (c$, "setZPos",
  function (i, flag, TF) {
 if (this.offsets == null || i >= this.offsets.length) {
 if (!TF) return;
-this.offsets = JU.AU.ensureLengthI (this.offsets, i + 1);
+this.offsets = JU.AU.ensureLengthI (this.offsets, this.ac);
 }this.offsets[i] = JV.JC.setZPosition (this.offsets[i], TF ? flag : 0);
 }, "~N,~N,~B");
 Clazz_defineMethod (c$, "setFont", 
  function (i, fid) {
 if (this.fids == null || i >= this.fids.length) {
 if (fid == this.zeroFontId) return;
-this.fids = JU.AU.ensureLengthByte (this.fids, i + 1);
+this.fids = JU.AU.ensureLengthByte (this.fids, this.ac);
 }this.fids[i] = fid;
 this.bsFontSet.set (i);
 var text = this.getLabel (i);
@@ -1950,7 +1979,6 @@ this.offset = 0;
 this.textAlign = 0;
 this.pointer = 0;
 this.zSlab = -2147483648;
-this.zCutoff = 0;
 this.zBox = 0;
 this.boxXY = null;
 this.scalePixelsPerMicron = 0;
@@ -1970,7 +1998,6 @@ var labelStrings = labels.strings;
 var fids = labels.fids;
 var offsets = labels.offsets;
 if (labelStrings == null) return false;
-this.setZcutoff ();
 var atoms = this.ms.at;
 var backgroundColixContrast = this.vwr.cm.colixBackgroundContrast;
 var backgroundColor = this.vwr.getBackgroundArgb ();
@@ -1996,7 +2023,6 @@ this.textAlign = JV.JC.getAlignment (this.offset);
 this.isAbsolute = JV.JC.isOffsetAbsolute (this.offset);
 this.pointer = JV.JC.getPointer (this.offset);
 this.zSlab = this.atom.sZ - Clazz_doubleToInt (this.atom.sD / 2) - 3;
-if (this.zSlab > this.zCutoff) continue;
 if (this.zSlab < 1) this.zSlab = 1;
 this.zBox = this.zSlab;
 if (labelsGroup) {
@@ -2022,10 +2048,6 @@ this.boxXY[1] /= 2;
 }
 return false;
 });
-Clazz_defineMethod (c$, "setZcutoff", 
-function () {
-this.zCutoff = (this.tm.zShadeEnabled ? this.tm.zSlabValue : 2147483647);
-});
 Clazz_defineMethod (c$, "renderLabelOrMeasure", 
 function (text, label) {
 var newText = false;
@@ -2114,7 +2136,6 @@ if (!this.isExport && this.m != null && (this.count = this.m.count) != 0) this.r
 if (!this.vwr.getBoolean (603979926)) return false;
 var showMeasurementLabels = this.vwr.getBoolean (603979879);
 measures.setVisibilityInfo ();
-this.setZcutoff ();
 for (var i = measures.measurementCount; --i >= 0; ) {
 this.m = measures.measurements.get (i);
 if (!this.m.isVisible || !this.m.$isValid || (this.count = this.m.count) == 1 && this.m.traceX == -2147483648) continue;
@@ -2209,7 +2230,7 @@ return;
 }var zA = a.sZ - a.sD - 10;
 var zB = b.sZ - b.sD - 10;
 var radius = this.drawLine (a.sX, a.sY, zA, b.sX, b.sY, zB, this.mad);
-if (s == null || zB >= this.zCutoff) return;
+if (s == null) return;
 if (this.mad > 0) radius <<= 1;
 var z = Clazz_doubleToInt ((zA + zB) / 2);
 if (z < 1) z = 1;
@@ -2232,7 +2253,7 @@ var zB = b.sZ - zOffset;
 var zC = c.sZ - c.sD - 10;
 var radius = this.drawLine (a.sX, a.sY, zA, b.sX, b.sY, zB, this.mad);
 radius += this.drawLine (b.sX, b.sY, zB, c.sX, c.sY, zC, this.mad);
-if (s == null || zB >= this.zCutoff) return;
+if (s == null) return;
 radius = Clazz_doubleToInt ((radius + 1) / 2);
 if (this.m.value > 175) {
 if (this.m.text == null) {
@@ -2291,7 +2312,7 @@ var radius = this.drawLine (a.sX, a.sY, zA, b.sX, b.sY, zB, this.mad);
 radius += this.drawLine (b.sX, b.sY, zB, c.sX, c.sY, zC, this.mad);
 radius += this.drawLine (c.sX, c.sY, zC, d.sX, d.sY, zD, this.mad);
 var zLabel = Clazz_doubleToInt ((zA + zB + zC + zD) / 4);
-if (s == null || zLabel >= this.zCutoff) return;
+if (s == null) return;
 radius /= 3;
 if (this.m.text == null) {
 this.g3d.setC (this.labelColix);
@@ -2347,7 +2368,6 @@ var echo = this.shape;
 var scalePixelsPerMicron = (this.vwr.getBoolean (603979845) ? this.vwr.getScalePixelsPerAngstrom (true) * 10000 : 0);
 this.imageFontScaling = this.vwr.imageFontScaling;
 var haveTranslucent = false;
-this.setZcutoff ();
 for (var t, $t = echo.objects.values ().iterator (); $t.hasNext () && ((t = $t.next ()) || true);) {
 if (!t.visible || t.hidden) {
 continue;
@@ -2360,8 +2380,7 @@ t.setXYZs (this.pt0i.x, this.pt0i.y, this.pt0i.z, this.pt0i.z);
 var z = this.vwr.tm.zValueFromPercent (t.movableZPercent % 1000);
 if (t.valign == 4 && Math.abs (t.movableZPercent) >= 1000) z = this.pt0i.z - this.vwr.tm.zValueFromPercent (0) + z;
 t.setZs (z, z);
-}if (t.zSlab >= this.zCutoff) continue;
-if (t.pointerPt == null) {
+}if (t.pointerPt == null) {
 t.pointer = 0;
 } else {
 t.pointer = 1;
@@ -2472,6 +2491,7 @@ return (this.vwr.ms.isJmolDataFrameForModel (atom.mi) && label.equals ("%U") ? "
 ,Clazz.getInheritedLevel
 ,Clazz.getParamsType
 ,Clazz.isAF
+,Clazz.isAB
 ,Clazz.isAI
 ,Clazz.isAS
 ,Clazz.isASS
